import random
import numpy as np
import torch
import logging
import os
import sys
import time
import pathlib
import math
import advertorch

# sys.path.append("../../")

def makedirs(dirname):
    pathlib.Path(dirname).mkdir(parents=True, exist_ok=True)


def get_logger(logpath, displaying=True, saving=True, debug=False):
    logger = logging.getLogger()
    if debug:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logger.setLevel(level)
    if saving:
        info_file_handler = logging.FileHandler(logpath, mode="a")
        info_file_handler.setLevel(level)
        logger.addHandler(info_file_handler)
    if displaying:
        console_handler = logging.StreamHandler()
        console_handler.setLevel(level)
        logger.addHandler(console_handler)
    # write info message
    logger.info('\n\n------ ******* ------ New Log ------ ******* ------')
    return logger


class get_epoch_logger():
    def __init__(self):
        self.epochs = []
        self.results = []
        self.best_epoch = 0; self.best_result = 0
        
    def append_results(self, list):
        self.epochs.append(list[0])
        self.results.append(list[1])
    
    def update_best_epoch(self):
 
        if self.results[-1] >= self.best_result:
            self.best_epoch = self.epochs[-1]
            self.best_result = self.results[-1]
        
        message = 'Best result @ {:03d}, {} \n'.format(self.best_epoch, self.best_result)
        return self.best_epoch, message
    
    def update_best_epoch_to_logger(self, logger):
            
        if self.results[-1] >= self.best_result:
            self.best_epoch = self.epochs[-1]
            self.best_result = self.results[-1]
        
        logger.info('Best result @ {:03d}, {} \n'.format(self.best_epoch, self.best_result))
        return self.best_epoch

class timer():
    def __init__(self):
        # print("Timer initialized @ "+ time.strftime('%Y-%m-%d-%H:%M:%S'))
        self.tic()
        
    def tic(self):
        self.t0 = time.time()
        # print(time.strftime('%Y-%m-%d-%H:%M:%S'))
        
    def toc(self, restart=True):
        diff = time.time()-self.t0
        if restart: self.t0 = time.time()
        return diff


def get_benchmark_sys_info():
    rval = "#\n#\n"
    rval += ("# Automatically generated benchmark report "
             "(screen print of running this file)\n#\n")
    uname = os.uname()
    rval += "# sysname: {}\n".format(uname.sysname)
    rval += "# release: {}\n".format(uname.release)
    rval += "# version: {}\n".format(uname.version)
    rval += "# machine: {}\n".format(uname.machine)
    rval += "# python: {}.{}.{}\n".format(
        sys.version_info.major,
        sys.version_info.minor,
        sys.version_info.micro)
    rval += "# torch: {}\n".format(torch.__version__)
    rval += "# advertorch: {}\n".format(advertorch.__version__)
    return rval


def count_parameters(model):
    return sum(p.numel() for p in model.parameters() if p.requires_grad)
